@@ -22,12 +22,14 @@ class ShellLab(Mcush.Mcush):
                 self.led(i, False)
 
     def daq( self, cmd, index=None, value=None ):
-        cmd = 'daq -c %s'% cmd
+        command = 'daq'
+        if cmd is not None:
+            command += ' -c %s'% cmd
         if index is not None:
-            cmd += ' -i %d'% index
+            command += ' -i %d'% index
         if value is not None:
-            cmd += ' -v %d'% value
-        return self.writeCommand( cmd )
+            command += ' -v %d'% value
+        return self.writeCommand( command )
 
     def daq_init( self, freq=None, length=None, channels=[], channel_mask=None ):
         if freq is not None:
@@ -81,15 +83,17 @@ class ShellLab(Mcush.Mcush):
     daqDone = daq_done
     daqRead = daq_read
 
-    def measure( self, cmd, index=None, value=None, fvalue=None ):
-        cmd = 'measure -c %s'% cmd
+    def measure( self, cmd=None, index=None, value=None, fvalue=None ):
+        command = 'measure'
+        if cmd is not None:
+            command += ' -c %s'% cmd
         if index is not None:
-            cmd += ' -i %d'% index
+            command += ' -i %d'% index
         if value is not None:
-            cmd += ' -v %d'% value
+            command += ' -v %d'% value
         if fvalue is not None:
-            cmd += ' -V %f'% fvalue
-        return self.writeCommand( cmd )
+            command += ' -V %f'% fvalue
+        return self.writeCommand( command )
 
     def measure_start( self ):
         self.measure( 'start' )
@@ -151,7 +155,7 @@ class ShellLabLamp(Mcush.Mcush):
         if freq is not None:
             cmd += ' -f %d'% freq
         return self.writeCommand(cmd)
- 
+
     def reset( self, lamp_freq=1, alarm_freq=1, mode=None ):
         self.lamp( 0, freq=lamp_freq, mode=mode )
         self.alarm( 0, freq=alarm_freq )
@@ -176,8 +180,8 @@ class ShellLabLamp(Mcush.Mcush):
         self.lamp( c, freq=freq, count=count ) 
 
 
-class ShellLabStrap(Mcush.Mcush):
-    DEFAULT_NAME = 'ShellLabStrap'
+class ShellLabStrip(Mcush.Mcush):
+    DEFAULT_NAME = 'ShellLabStrip'
     DEFAULT_IDN = re_compile( 'ShellLab-L2[a-zA-Z]*,([0-9]+\.[0-9]+.*)' )
 
     def __init__( self, *args, **kwargs ):
@@ -206,6 +210,8 @@ class ShellLabStrap(Mcush.Mcush):
             cmd += ' -C %d'% count
         return self.writeCommand(cmd)
 
+    strip = strap
+
     def reset( self, freq=1 ):
         self.strap( 0, freq=freq )
 
@@ -221,9 +227,9 @@ class ShellLabStrap(Mcush.Mcush):
         pass
 
     
-class ShellLabStaticStrap(ShellLabStrap):
+class ShellLabStaticStrip(ShellLabStrip):
     def __init__( self, *args, **kwargs ):
-        ShellLabStrap.__init__(self, *args, **kwargs)
+        ShellLabStrip.__init__(self, *args, **kwargs)
         setting = Utils.parseKeyValueLines(self.strap())
         if float(setting['freq']) > 0:
             self.strap( freq=0 )
@@ -257,7 +263,43 @@ class ShellLabStaticStrap(ShellLabStrap):
             
     setColor = color
     setBrightness = brightness
-   
+
+# for compatibility
+ShellLabStrap = ShellLabStrip
+ShellLabStaticStrap = ShellLabStaticStrip
+
+
+class ShellLabSpotLight(Mcush.Mcush):
+    DEFAULT_NAME = 'ShellLabSpotLight'
+    DEFAULT_IDN = re_compile( 'ShellLab-SpotLight,([0-9]+\.[0-9]+.*)' )
+
+    def light( self, index=None, power=None, freq=None, count=None, reset=None, sync=None ):
+        cmd = 's'
+        if index is not None:
+            cmd += ' -i %d'% index
+        if power is not None:
+            cmd += ' -p %d'% power
+        if freq is not None:
+            cmd += ' -f %s'% freq
+        if count is not None:
+            cmd += ' -c %d'% count
+        if reset:
+            cmd += ' -r'
+        if sync:
+            cmd += ' -s'
+        return self.writeCommand(cmd)
+
+    def reset( self ):
+        self.light( reset=True )
+
+    def sync( self ):
+        self.light( sync=True )
+
+
+class ShellLabTowerLamp(ShellLabSpotLight):
+    # TODO: optimize tower lamp APIs 
+    pass
+
 
 class ShellLabCAN(ShellLab):
     DEFAULT_NAME = 'ShellLabCAN'
@@ -318,7 +360,7 @@ class ShellLabCANopen(ShellLabCAN):
 
     def startNode( self, id ):
         self.writeNMTCommand( NMT_START, id )
- 
+
     def stopNode( self, id ):
         self.writeNMTCommand( NMT_STOP, id )
 
@@ -353,7 +395,7 @@ class ShellLabCANopen(ShellLabCAN):
 
     # Object dictionaries
     def readObject( self, id, index, subindex ):
-        self.canWrite( RSDO+id, [0x5F, index&0xFF, (index>>8)&0xFF, subindex, 0, 0, 0, 0] )
+        self.canWrite( RSDO+id, [0x40, index&0xFF, (index>>8)&0xFF, subindex, 0, 0, 0, 0] )
         bytes_required = None
         t0 = time.time()
         responsed = None
@@ -641,7 +683,7 @@ class ShellLabSegmentDisplay(Mcush.Mcush):
 
     def int(self, val, idx=None):
         self.disp('int', idx=idx, val=val)
- 
+
     def msg(self, msg, idx=None):
         self.disp(idx=idx, msg=msg)
 
@@ -685,7 +727,20 @@ class ShellLabSensor(Mcush.Mcush):
             command += ' -v%d'% val
         return self.writeCommand( command ) 
 
- 
+
+class ShellLabSensorT(ShellLabSensor):
+    '''DS18B20 based temperature sensor'''
+    DEFAULT_NAME = 'ShellLab-Sensor-T'
+    DEFAULT_IDN = re_compile( 'ShellLab-Sensor-T,([0-9]+\.[0-9]+.*)' )
+
+    def getT( self ):
+        ret = self.measure()
+        k,v = ret[0].split(':')
+        if k == 'T':
+            return float(v)
+        return None 
+
+
 class ShellLabSensorTH(ShellLabSensor):
     '''SHT30 based temperature/humidity sensor'''
     DEFAULT_NAME = 'ShellLab-Sensor-TH'
